//
//  AbstractAjaxAction.java
//  TestComposants
//
//  Created by Lionel GUILLAUME on 09/03/06.
//  Copyright (c) 2006 __MyCompanyName__. All rights reserved.
//

import com.webobjects.foundation.*;
import com.webobjects.appserver.*;
import com.webobjects.eocontrol.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import java.io.*;

public abstract class AbstractAjaxDA extends WODirectAction {
		
	protected WORequest request;
	private Document document;
	
    public AbstractAjaxDA(WORequest request) {
        super(request);
		this.request = request;
    }

	public WOSession session() {
		WOSessionStore sessionStore = application().sessionStore();
		String sessionId = this.getSessionIDForRequest(request);
		WOSession woSession = sessionStore.restoreSessionWithID(sessionId, request);
		return woSession;
	}
	
	public WOApplication application () {
		return WOApplication.application();
	}
	
	public WOResponse responseWithObject (Object o) {
		WOContext context = application().createContextForRequest(request);
		WOResponse response = application().createResponseInContext(context);
		//response.setHeader ("text/xml", "Content-Type");
		response.setHeader("text/xml; charset=iso-8859-15", "content-type"); // Correction pour les accents
		
		try {
			//Output the XML
			Document doc = fundation2plist(o);
			//set up a transformer
			TransformerFactory transfac = TransformerFactory.newInstance();
			Transformer trans = transfac.newTransformer();
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
			trans.setOutputProperty(OutputKeys.INDENT, "yes");
			
			//create string from xml tree
			StringWriter sw = new StringWriter();
			StreamResult result = new StreamResult(sw);
			DOMSource source = new DOMSource(doc);
			trans.transform(source, result);
			String xmlString = sw.toString();
			response.appendContentString  ( xmlString );
				
			//print xml
			//System.out.println("Here's the xml:\n\n" + xmlString);
		} catch ( Exception e ) {
			e.printStackTrace();
		}		
		return response;
	}
	
	private Document fundation2plist (Object o) {
		DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
			document = docBuilder.newDocument();
		} catch (Exception e) {
			System.err.println ("Erreur Document fundation2plist (Object o) "+e);
			return null;
		}
		Element rootElement = document.createElement("plist");
		document.appendChild(rootElement);
		
		rootElement.appendChild(object2Element (o));
		return document;
	}
	
	private Element object2Element (Object o) {
		if (o == null) {
			o = "";
		}
		else if (o instanceof String) {
			return string2Element((String)o);
		}
		else if (o instanceof Integer) {
			return integer2Element((Integer)o);
		}
		else if (o instanceof Number) {
			return number2Element((Number)o);
		}
		else if (o instanceof Boolean) {
			return boolean2Element((Boolean)o);
		}
		else if (o instanceof NSDictionary) {
			return nsDictionary2Element((NSDictionary)o);
		}
		else if (o instanceof NSArray) {
			return nsArray2Element((NSArray)o);
		}
		else if (o instanceof NSTimestamp) {
			return nsTimestamp2Element((NSTimestamp)o);
		}
		else if (o instanceof EOEnterpriseObject) {
			return eoEnterpriseObject2Element((EOEnterpriseObject)o);
		}
		return document.createElement("string");
	}
	
	private Element string2Element (String s) {
		Element element = document.createElement("string");
		Text text = document.createTextNode(s);
		element.appendChild(text);
		return element;
	}
	
	private Element integer2Element (Integer i) {
		Element element = document.createElement("integer");
		Text text = document.createTextNode(i.toString());
		element.appendChild(text);
		return element;
	}
	
	private Element number2Element (Number n) {
		Element element = document.createElement("real");
		Text text = document.createTextNode(n.toString());
		element.appendChild(text);
		return element;
	}
	
	private Element boolean2Element (Boolean b) {
		if (b.equals(Boolean.TRUE)) 
			return document.createElement("true");
		return document.createElement("false");
	}
	
	private Element nsDictionary2Element (NSDictionary dict) {
		Element element = document.createElement("dict");
		NSArray allKeys = dict.allKeys();
		String key = null;
		Element keyElement = null;
		Object o = null;
		for (int i=0; i<allKeys.count(); i++) {
			key = (String)allKeys.objectAtIndex(i);
			keyElement = document.createElement("key");
			Text text = document.createTextNode(key);
			keyElement.appendChild(text);
			element.appendChild(keyElement);
			o = dict.valueForKey (key);
			element.appendChild( object2Element(o));
		}
		return element;
	}
	
	private Element nsArray2Element (NSArray array) {
		Element element = document.createElement("array");
		for (int i=0; i<array.count(); i++) {
			element.appendChild( object2Element( array.objectAtIndex(i) ) );
		}
		return element;
	}
	
	private Element nsTimestamp2Element (NSTimestamp time) {
		Element element = document.createElement("date");
		Text text = document.createTextNode(time.toString());
		element.appendChild(text);
		return element;
	}
	
	private Element eoEnterpriseObject2Element (EOEnterpriseObject eObject) {
		Element element = document.createElement("dict");
		NSArray allKeys = eObject.attributeKeys();
		String key = null;
		Element keyElement = null;
		Object o = null;
		for (int i=0; i<allKeys.count(); i++) {
			key = (String)allKeys.objectAtIndex(i);
			keyElement = document.createElement("key");
			Text text = document.createTextNode(key);
			keyElement.appendChild(text);
			element.appendChild(keyElement);
			o = eObject.valueForKey (key);
			if (o!=null) {
				element.appendChild( object2Element(o));
			}
		}
		return element;
	}
	
}
